# Generated by Django 2.2.18 on 2021-03-11 20:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0120_auto_20210221_2206'),
    ]

    operations = [
        migrations.AlterField(
            model_name='negative',
            name='teleconverter',
            field=models.ForeignKey(blank=True, help_text='Teleconverter used when taking this negative', null=True, on_delete=django.db.models.deletion.CASCADE, to='schema.Teleconverter'),
        ),
    ]



# pylint: disable=no-member

from django.views import generic
from django.views.generic.edit import CreateView, UpdateView
from django.views.generic import TemplateView, ListView
from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_control
from django.db.models import Sum
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth import get_user_model
from django.apps import apps
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django_tables2 import SingleTableView
from django_tables2.views import SingleTableMixin
from django_filters.views import FilterView
from watson.views import SearchMixin
from taggit.models import Tag
from dal import autocomplete

from schema.models import Accessory, Archive, Battery, BulkFilm, Camera, CameraModel, Developer, Enlarger, EnlargerModel, FilmStock, Filter
from schema.models import Flash, FlashModel, Format, Lens, LensModel, Manufacturer
from schema.models import Mount, MountAdapter, NegativeSize, Order, PaperStock, Person, Print
from schema.models import Process, Scan, Negative, Film, Teleconverter, TeleconverterModel, Toner

from schema.tables import AccessoryTable, ArchiveTable, BatteryTable, BulkFilmTable, CameraTable, CameraModelTable, DeveloperTable, EnlargerTable, EnlargerModelTable, FilmStockTable, FilterTable
from schema.tables import FlashTable, FlashModelTable, FormatTable, LensTable, LensModelTable, ManufacturerTable
from schema.tables import MountTable, MountAdapterTable, NegativeSizeTable, OrderTable, PaperStockTable, PersonTable, PrintTable
from schema.tables import ProcessTable, ScanTable, NegativeTable, FilmTable, TeleconverterTable, TeleconverterModelTable, TonerTable

from schema.forms import AccessoryForm, ArchiveForm, BatteryForm, BulkFilmForm, CameraForm, CameraSellForm, CameraModelForm, DeveloperForm, EnlargerForm, EnlargerModelForm, FilmStockForm, FilterForm
from schema.forms import FlashForm, FlashModelForm, FormatForm, LensForm, LensSellForm, LensModelForm, ManufacturerForm
from schema.forms import MountForm, MountAdapterForm, NegativeSizeForm, OrderForm, PaperStockForm, PersonForm, PrintForm, PrintArchiveForm
from schema.forms import ProcessForm, ScanForm, NegativeForm, FilmForm, FilmAddForm, FilmLoadForm, FilmDevelopForm, FilmArchiveForm, TeleconverterForm, TeleconverterModelForm, TonerForm

from schema.filters import AccessoryFilter, BatteryFilter, BulkFilmFilter, CameraFilter, CameraModelFilter, DeveloperFilter
from schema.filters import EnlargerFilter, EnlargerModelFilter, FilmFilter, FilmStockFilter, FlashFilter, FlashModelFilter, LensFilter, LensModelFilter
from schema.filters import MountAdapterFilter, MountFilter, NegativeFilter, OrderFilter, PaperStockFilter, PrintFilter
from schema.filters import TeleconverterFilter, TeleconverterModelFilter, TonerFilter

from schema.formhelpers import AccessoryFormHelper, BatteryFormHelper, BulkFilmFormHelper, CameraFormHelper, CameraModelFormHelper
from schema.formhelpers import DeveloperFormHelper, EnlargerFormHelper, EnlargerModelFormHelper, FilmFormHelper, FilmStockFormHelper, FlashFormHelper, FlashModelFormHelper
from schema.formhelpers import LensFormHelper, LensModelFormHelper, MountAdapterFormHelper, MountFormHelper, NegativeFormHelper
from schema.formhelpers import OrderFormHelper, PaperStockFormHelper, PrintFormHelper, TeleconverterFormHelper, TeleconverterModelFormHelper, TonerFormHelper

from .funcs import to_dict

# Custom class for filtered list views in table format


class PagedFilteredTableView(SingleTableMixin, FilterView):
    filterset_class = None
    formhelper_class = None
    context_filter_name = 'filter'
    form_method = 'POST'
    template_name = 'list.html'
    paginate_by = 25

    # pylint: disable=not-callable,attribute-defined-outside-init
    def get_queryset(self):
        qs = super().get_queryset()
        self.filter = self.filterset_class(self.request.GET, queryset=qs)
        self.filter.form.helper = self.formhelper_class()
        return self.filter.qs

 #   def get_table(self, **kwargs):
 #       table = super().get_table()
 #       RequestConfig(self.request, paginate={'page': self.kwargs['page'],
 #                           "per_page": self.paginate_by}).configure(table)
 #       return table

    def get_context_data(self, **kwargs):
        context = super().get_context_data()
        context[self.context_filter_name] = self.filter
        return context

# Custom class for displaying a list view in table format


class SingleTableListView(SingleTableView):
    template_name = 'list.html'
    paginate_by = 25

@method_decorator(cache_control(private=True), name='dispatch')
class AccessoryList(LoginRequiredMixin, PagedFilteredTableView):
    model = Accessory
    table_class = AccessoryTable
    filterset_class = AccessoryFilter
    formhelper_class = AccessoryFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class AccessoryDetail(LoginRequiredMixin, generic.DetailView):
    model = Accessory

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Accessory, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class AccessoryCreate(LoginRequiredMixin, CreateView):
    model = Accessory
    form_class = AccessoryForm
    template_name = 'create.html'


class AccessoryUpdate(LoginRequiredMixin, UpdateView):
    model = Accessory
    form_class = AccessoryForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Accessory, owner=self.request.user, id_owner=self.kwargs['id_owner'])


@method_decorator(cache_control(private=True), name='dispatch')
class ArchiveList(LoginRequiredMixin, SingleTableListView):
    model = Archive
    table_class = ArchiveTable


@method_decorator(cache_control(private=True), name='dispatch')
class ArchiveDetail(LoginRequiredMixin, generic.DetailView):
    model = Archive

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Archive, owner=self.request.user, id_owner=self.kwargs['id_owner'])


@method_decorator(cache_control(private=True), name='dispatch')
class ArchivePrint(LoginRequiredMixin, generic.DetailView):
    model = Archive
    template_name = 'schema/archive_print.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Archive, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class ArchiveCreate(LoginRequiredMixin, CreateView):
    model = Archive
    form_class = ArchiveForm
    template_name = 'create.html'


class ArchiveUpdate(LoginRequiredMixin, UpdateView):
    model = Archive
    form_class = ArchiveForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Archive, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class BatteryList(PagedFilteredTableView):
    model = Battery
    table_class = BatteryTable
    filterset_class = BatteryFilter
    formhelper_class = BatteryFormHelper


class BatteryDetail(generic.DetailView):
    model = Battery

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()
        return context


class BatteryCreate(LoginRequiredMixin, CreateView):
    model = Battery
    form_class = BatteryForm
    template_name = 'create.html'


class BatteryUpdate(LoginRequiredMixin, UpdateView):
    model = Battery
    form_class = BatteryForm
    template_name = 'update.html'


@method_decorator(cache_control(private=True), name='dispatch')
class BulkFilmList(LoginRequiredMixin, PagedFilteredTableView):
    model = BulkFilm
    table_class = BulkFilmTable
    filterset_class = BulkFilmFilter
    formhelper_class = BulkFilmFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class BulkFilmDetail(LoginRequiredMixin, generic.DetailView):
    model = BulkFilm

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(BulkFilm, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class BulkFilmCreate(LoginRequiredMixin, CreateView):
    model = BulkFilm
    form_class = BulkFilmForm
    template_name = 'create.html'


class BulkFilmUpdate(LoginRequiredMixin, UpdateView):
    model = BulkFilm
    form_class = BulkFilmForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(BulkFilm, owner=self.request.user, id_owner=self.kwargs['id_owner'])


@method_decorator(cache_control(private=True), name='dispatch')
class CameraList(LoginRequiredMixin, PagedFilteredTableView):
    model = Camera
    table_class = CameraTable
    filterset_class = CameraFilter
    formhelper_class = CameraFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class CameraDetail(LoginRequiredMixin, generic.DetailView):
    model = Camera

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Camera, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class CameraCreate(LoginRequiredMixin, CreateView):
    model = Camera
    form_class = CameraForm
    template_name = 'create.html'

    def get_initial(self):
        initial = super().get_initial()
        if 'cameramodel' in self.request.GET:
            initial.update({'cameramodel': self.request.GET['cameramodel']})
        return initial


class CameraUpdate(LoginRequiredMixin, UpdateView):
    model = Camera
    form_class = CameraForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Camera, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class CameraSell(LoginRequiredMixin, UpdateView):
    model = Camera
    form_class = CameraSellForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Camera, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class CameraModelList(PagedFilteredTableView):
    model = CameraModel
    table_class = CameraModelTable
    filterset_class = CameraModelFilter
    formhelper_class = CameraModelFormHelper

    # Pass request object to table context
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        table = self.table_class
        table.request = self.request
        return context


class CameraModelDetail(generic.DetailView):
    model = CameraModel

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        # Find similar objects of the same type
        similarobjects = self.get_object().tags.similar_objects()
        items = []
        for index, item in zip(range(10), similarobjects): # pylint: disable=unused-variable
            if type(item) == type(self.get_object()):  # pylint: disable=unidiomatic-typecheck
                detailitem = get_object_or_404(type(item), pk=item.pk)
                items.append(detailitem)
        context['related'] = items

        if self.request.user.is_authenticated:
            context['mine'] = self.get_object().camera_set.filter(
                owner=self.request.user)

        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()

        return context


class CameraModelCreate(LoginRequiredMixin, CreateView):
    model = CameraModel
    form_class = CameraModelForm
    template_name = 'create.html'

    def get_initial(self):
        initial = super().get_initial()
        if 'clone' in self.request.GET:
            # Retrieve original object
            original = CameraModel.objects.get(slug=self.request.GET['clone'])

            # Copy the original object to the initial state of the new one
            initial = to_dict(original)

            # Set disambiguation
            initial['disambiguation'] = 'Clone of ' + str(original)

        return initial


class CameraModelUpdate(LoginRequiredMixin, UpdateView):
    model = CameraModel
    form_class = CameraModelForm
    template_name = 'update.html'


class DeveloperList(PagedFilteredTableView):
    model = Developer
    table_class = DeveloperTable
    filterset_class = DeveloperFilter
    formhelper_class = DeveloperFormHelper


class DeveloperDetail(generic.DetailView):
    model = Developer

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # Find similar objects of the same type
        similarobjects = self.get_object().tags.similar_objects()
        items = []
        for index, item in zip(range(10), similarobjects): # pylint: disable=unused-variable
            if type(item) == type(self.get_object()):  # pylint: disable=unidiomatic-typecheck
                detailitem = get_object_or_404(type(item), pk=item.pk)
                items.append(detailitem)
        context['related'] = items

        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()

        return context


class DeveloperCreate(LoginRequiredMixin, CreateView):
    model = Developer
    form_class = DeveloperForm
    template_name = 'create.html'


class DeveloperUpdate(LoginRequiredMixin, UpdateView):
    model = Developer
    form_class = DeveloperForm
    template_name = 'update.html'


class EnlargerModelList(PagedFilteredTableView):
    model = EnlargerModel
    table_class = EnlargerModelTable
    filterset_class = EnlargerModelFilter
    formhelper_class = EnlargerModelFormHelper


class EnlargerModelDetail(generic.DetailView):
    model = EnlargerModel

class EnlargerModelCreate(LoginRequiredMixin, CreateView):
    model = EnlargerModel
    form_class = EnlargerModelForm
    template_name = 'create.html'


class EnlargerModelUpdate(LoginRequiredMixin, UpdateView):
    model = EnlargerModel
    form_class = EnlargerModelForm
    template_name = 'update.html'

@method_decorator(cache_control(private=True), name='dispatch')
class EnlargerList(LoginRequiredMixin, PagedFilteredTableView):
    model = Enlarger
    table_class = EnlargerTable
    filterset_class = EnlargerFilter
    formhelper_class = EnlargerFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class EnlargerDetail(LoginRequiredMixin, generic.DetailView):
    model = Enlarger

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Enlarger, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class EnlargerCreate(LoginRequiredMixin, CreateView):
    model = Enlarger
    form_class = EnlargerForm
    template_name = 'create.html'


class EnlargerUpdate(LoginRequiredMixin, UpdateView):
    model = Enlarger
    form_class = EnlargerForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Enlarger, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class FilmStockList(PagedFilteredTableView):
    model = FilmStock
    table_class = FilmStockTable
    filterset_class = FilmStockFilter
    formhelper_class = FilmStockFormHelper


class FilmStockDetail(generic.DetailView):
    model = FilmStock

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # Find similar objects of the same type
        similarobjects = self.get_object().tags.similar_objects()
        items = []
        for index, item in zip(range(10), similarobjects): # pylint: disable=unused-variable
            if type(item) == type(self.get_object()):  # pylint: disable=unidiomatic-typecheck
                detailitem = get_object_or_404(type(item), pk=item.pk)
                items.append(detailitem)
        context['related'] = items

        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()

        return context


class FilmStockCreate(LoginRequiredMixin, CreateView):
    model = FilmStock
    form_class = FilmStockForm
    template_name = 'create.html'


class FilmStockUpdate(LoginRequiredMixin, UpdateView):
    model = FilmStock
    form_class = FilmStockForm
    template_name = 'update.html'


class FilterList(SingleTableListView):
    model = Filter
    table_class = FilterTable


class FilterDetail(generic.DetailView):
    model = Filter


class FilterCreate(LoginRequiredMixin, CreateView):
    model = Filter
    form_class = FilterForm
    template_name = 'create.html'


class FilterUpdate(LoginRequiredMixin, UpdateView):
    model = Filter
    form_class = FilterForm
    template_name = 'update.html'


class FlashModelList(PagedFilteredTableView):
    model = FlashModel
    table_class = FlashModelTable
    filterset_class = FlashModelFilter
    formhelper_class = FlashModelFormHelper


class FlashModelDetail(generic.DetailView):
    model = FlashModel


class FlashModelCreate(LoginRequiredMixin, CreateView):
    model = FlashModel
    form_class = FlashModelForm
    template_name = 'create.html'


class FlashModelUpdate(LoginRequiredMixin, UpdateView):
    model = FlashModel
    form_class = FlashModelForm
    template_name = 'update.html'


@method_decorator(cache_control(private=True), name='dispatch')
class FlashList(LoginRequiredMixin, PagedFilteredTableView):
    model = Flash
    table_class = FlashTable
    filterset_class = FlashFilter
    formhelper_class = FlashFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class FlashDetail(LoginRequiredMixin, generic.DetailView):
    model = Flash

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Flash, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class FlashCreate(LoginRequiredMixin, CreateView):
    model = Flash
    form_class = FlashForm
    template_name = 'create.html'


class FlashUpdate(LoginRequiredMixin, UpdateView):
    model = Flash
    form_class = FlashForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Flash, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class FormatList(SingleTableListView):
    model = Format
    table_class = FormatTable


class FormatDetail(generic.DetailView):
    model = Format

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()
        return context


class FormatCreate(LoginRequiredMixin, CreateView):
    model = Format
    form_class = FormatForm
    template_name = 'create.html'


class FormatUpdate(LoginRequiredMixin, UpdateView):
    model = Format
    form_class = FormatForm
    template_name = 'update.html'


@method_decorator(cache_control(private=True), name='dispatch')
class LensList(LoginRequiredMixin, PagedFilteredTableView):
    model = Lens
    table_class = LensTable
    filterset_class = LensFilter
    formhelper_class = LensFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class LensDetail(LoginRequiredMixin, generic.DetailView):
    model = Lens

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Lens, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class LensCreate(LoginRequiredMixin, CreateView):
    model = Lens
    form_class = LensForm
    template_name = 'create.html'

    def get_initial(self):
        initial = super().get_initial()
        if 'lensmodel' in self.request.GET:
            initial.update({'lensmodel': self.request.GET['lensmodel']})
        return initial


class LensUpdate(LoginRequiredMixin, UpdateView):
    model = Lens
    form_class = LensForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Lens, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class LensSell(LoginRequiredMixin, UpdateView):
    model = Lens
    form_class = LensSellForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Lens, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class LensModelList(PagedFilteredTableView):
    model = LensModel
    table_class = LensModelTable
    filterset_class = LensModelFilter
    formhelper_class = LensModelFormHelper

    # Pass request object to table context
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        table = self.table_class
        table.request = self.request
        return context


class LensModelDetail(generic.DetailView):
    model = LensModel

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        # Find similar objects of the same type
        similarobjects = self.get_object().tags.similar_objects()
        items = []
        for index, item in zip(range(10), similarobjects): # pylint: disable=unused-variable
            if type(item) == type(self.get_object()):  # pylint: disable=unidiomatic-typecheck
                detailitem = get_object_or_404(type(item), pk=item.pk)
                items.append(detailitem)
        context['related'] = items

        if self.request.user.is_authenticated:
            context['mine'] = self.get_object().lens_set.filter(
                owner=self.request.user)

        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()

        return context


class LensModelCreate(LoginRequiredMixin, CreateView):
    model = LensModel
    form_class = LensModelForm
    template_name = 'create.html'

    def get_initial(self):
        initial = super().get_initial()
        if 'clone' in self.request.GET:
            # Retrieve original object
            original = LensModel.objects.get(slug=self.request.GET['clone'])

            # Copy the original object to the initial state of the new one
            initial = to_dict(original)

            # Set disambiguation
            initial['disambiguation'] = 'Clone of ' + str(original)

        return initial


class LensModelUpdate(LoginRequiredMixin, UpdateView):
    model = LensModel
    form_class = LensModelForm
    template_name = 'update.html'


class ManufacturerList(SingleTableListView):
    model = Manufacturer
    table_class = ManufacturerTable


class ManufacturerDetail(generic.DetailView):
    model = Manufacturer

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        # Find similar objects of the same type
        similarobjects = self.get_object().tags.similar_objects()
        items = []
        for index, item in zip(range(10), similarobjects): # pylint: disable=unused-variable
            if type(item) == type(self.get_object()):  # pylint: disable=unidiomatic-typecheck
                detailitem = get_object_or_404(type(item), pk=item.pk)
                items.append(detailitem)
        context['related'] = items

        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()

        return context


class ManufacturerCreate(LoginRequiredMixin, CreateView):
    model = Manufacturer
    form_class = ManufacturerForm
    template_name = 'create.html'


class ManufacturerUpdate(LoginRequiredMixin, UpdateView):
    model = Manufacturer
    form_class = ManufacturerForm
    template_name = 'update.html'


class MountList(PagedFilteredTableView):
    model = Mount
    table_class = MountTable
    filterset_class = MountFilter
    formhelper_class = MountFormHelper


class MountDetail(generic.DetailView):
    model = Mount

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        # Find similar objects of the same type
        similarobjects = self.get_object().tags.similar_objects()
        items = []
        for index, item in zip(range(10), similarobjects): # pylint: disable=unused-variable
            if type(item) == type(self.get_object()):  # pylint: disable=unidiomatic-typecheck
                detailitem = get_object_or_404(type(item), pk=item.pk)
                items.append(detailitem)
        context['related'] = items

        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()

        return context


class MountCreate(LoginRequiredMixin, CreateView):
    model = Mount
    form_class = MountForm
    template_name = 'create.html'


class MountUpdate(LoginRequiredMixin, UpdateView):
    model = Mount
    form_class = MountForm
    template_name = 'update.html'


@method_decorator(cache_control(private=True), name='dispatch')
class MountAdapterList(LoginRequiredMixin, PagedFilteredTableView):
    model = MountAdapter
    table_class = MountAdapterTable
    filterset_class = MountAdapterFilter
    formhelper_class = MountAdapterFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class MountAdapterDetail(LoginRequiredMixin, generic.DetailView):
    model = MountAdapter

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(MountAdapter, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class MountAdapterCreate(LoginRequiredMixin, CreateView):
    model = MountAdapter
    form_class = MountAdapterForm
    template_name = 'create.html'


class MountAdapterUpdate(LoginRequiredMixin, UpdateView):
    model = MountAdapter
    form_class = MountAdapterForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(MountAdapter, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class NegativeSizeList(SingleTableListView):
    model = NegativeSize
    table_class = NegativeSizeTable


class NegativeSizeDetail(generic.DetailView):
    model = NegativeSize

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()
        return context


class NegativeSizeCreate(LoginRequiredMixin, CreateView):
    model = NegativeSize
    form_class = NegativeSizeForm
    template_name = 'create.html'


class NegativeSizeUpdate(LoginRequiredMixin, UpdateView):
    model = NegativeSize
    form_class = NegativeSizeForm
    template_name = 'update.html'


@method_decorator(cache_control(private=True), name='dispatch')
class OrderList(LoginRequiredMixin, PagedFilteredTableView):
    model = Order
    table_class = OrderTable
    filterset_class = OrderFilter
    formhelper_class = OrderFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class OrderDetail(LoginRequiredMixin, generic.DetailView):
    model = Order

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Order, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class OrderCreate(LoginRequiredMixin, CreateView):
    model = Order
    form_class = OrderForm
    template_name = 'create.html'


class OrderUpdate(LoginRequiredMixin, UpdateView):
    model = Order
    form_class = OrderForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Order, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class PaperStockList(PagedFilteredTableView):
    model = PaperStock
    table_class = PaperStockTable
    filterset_class = PaperStockFilter
    formhelper_class = PaperStockFormHelper


class PaperStockDetail(generic.DetailView):
    model = PaperStock

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        # Find similar objects of the same type
        similarobjects = self.get_object().tags.similar_objects()
        items = []
        for index, item in zip(range(10), similarobjects): # pylint: disable=unused-variable
            if type(item) == type(self.get_object()):  # pylint: disable=unidiomatic-typecheck
                detailitem = get_object_or_404(type(item), pk=item.pk)
                items.append(detailitem)
        context['related'] = items

        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()

        return context


class PaperStockCreate(LoginRequiredMixin, CreateView):
    model = PaperStock
    form_class = PaperStockForm
    template_name = 'create.html'


class PaperStockUpdate(LoginRequiredMixin, UpdateView):
    model = PaperStock
    form_class = PaperStockForm
    template_name = 'update.html'


@method_decorator(cache_control(private=True), name='dispatch')
class PersonList(LoginRequiredMixin, SingleTableListView):
    model = Person
    table_class = PersonTable

    def get_queryset(self):
        if self.request.user.is_authenticated:
            mystr = Person.objects.filter(owner=self.request.user)
        else:
            mystr = Person.objects.none()
        return mystr


@method_decorator(cache_control(private=True), name='dispatch')
class PersonDetail(LoginRequiredMixin, generic.DetailView):
    model = Person

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Person, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class PersonCreate(LoginRequiredMixin, CreateView):
    model = Person
    form_class = PersonForm
    template_name = 'create.html'


class PersonUpdate(LoginRequiredMixin, UpdateView):
    model = Person
    form_class = PersonForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Person, owner=self.request.user, id_owner=self.kwargs['id_owner'])


@method_decorator(cache_control(private=True), name='dispatch')
class PrintList(LoginRequiredMixin, PagedFilteredTableView):
    model = Print
    table_class = PrintTable
    filterset_class = PrintFilter
    formhelper_class = PrintFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class PrintDetail(LoginRequiredMixin, generic.DetailView):
    model = Print

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Print, owner=self.request.user, id_owner=self.kwargs['id_owner'])


@method_decorator(cache_control(private=True), name='dispatch')
class PrintPrint(LoginRequiredMixin, generic.DetailView):
    model = Print
    template_name = 'schema/print_print.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Print, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class PrintCreate(LoginRequiredMixin, CreateView):
    model = Print
    form_class = PrintForm
    template_name = 'create.html'

    def get_initial(self):
        initial = super().get_initial()
        if 'negative' in self.request.GET:
            initial.update({'negative': self.request.GET['negative']})
        return initial


class PrintUpdate(LoginRequiredMixin, UpdateView):
    model = Print
    form_class = PrintForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Print, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class PrintArchive(LoginRequiredMixin, UpdateView):
    model = Print
    form_class = PrintArchiveForm
    template_name = 'update.html'

# Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Print, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class ProcessList(SingleTableListView):
    model = Process
    table_class = ProcessTable


class ProcessDetail(generic.DetailView):
    model = Process


class ProcessCreate(LoginRequiredMixin, CreateView):
    model = Process
    form_class = ProcessForm
    template_name = 'create.html'


class ProcessUpdate(LoginRequiredMixin, UpdateView):
    model = Process
    form_class = ProcessForm
    template_name = 'update.html'


@method_decorator(cache_control(private=True), name='dispatch')
class ScanList(LoginRequiredMixin, SingleTableListView):
    model = Scan
    table_class = ScanTable

    def get_queryset(self):
        if self.request.user.is_authenticated:
            mystr = Scan.objects.filter(owner=self.request.user)
        else:
            mystr = Scan.objects.none()
        return mystr


@method_decorator(cache_control(private=True), name='dispatch')
class ScanDetail(LoginRequiredMixin, generic.DetailView):
    model = Scan

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Scan, owner=self.request.user, uuid=self.kwargs['uuid'])


class ScanCreate(LoginRequiredMixin, CreateView):
    model = Scan
    form_class = ScanForm
    template_name = 'create.html'


class ScanUpdate(LoginRequiredMixin, UpdateView):
    model = Scan
    form_class = ScanForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Scan, owner=self.request.user, uuid=self.kwargs['uuid'])


@method_decorator(cache_control(private=True), name='dispatch')
class NegativeList(LoginRequiredMixin, PagedFilteredTableView):
    model = Negative
    table_class = NegativeTable
    filterset_class = NegativeFilter
    formhelper_class = NegativeFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class NegativeDetail(LoginRequiredMixin, generic.DetailView):
    model = Negative

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Negative, owner=self.request.user, slug=self.kwargs['slug'])


class NegativeCreate(LoginRequiredMixin, CreateView):
    model = Negative
    form_class = NegativeForm
    template_name = 'create.html'

    def get_initial(self):
        initial = super().get_initial()
        if 'film' in self.request.GET:
            initial.update({'film': self.request.GET['film']})
        return initial


class NegativeUpdate(LoginRequiredMixin, UpdateView):
    model = Negative
    form_class = NegativeForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Negative, owner=self.request.user, slug=self.kwargs['slug'])


@method_decorator(cache_control(private=True), name='dispatch')
class FilmList(LoginRequiredMixin, PagedFilteredTableView):
    model = Film
    table_class = FilmTable
    filterset_class = FilmFilter
    formhelper_class = FilmFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class FilmDetail(LoginRequiredMixin, generic.DetailView):
    model = Film

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])


@method_decorator(cache_control(private=True), name='dispatch')
class FilmPrint(LoginRequiredMixin, generic.DetailView):
    model = Film
    template_name = 'schema/film_print.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class FilmCreate(LoginRequiredMixin, CreateView):
    model = Film
    form_class = FilmAddForm
    template_name = 'create.html'


class FilmUpdate(LoginRequiredMixin, UpdateView):
    model = Film
    form_class = FilmForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class FilmLoad(LoginRequiredMixin, UpdateView):
    model = Film
    form_class = FilmLoadForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class FilmDevelop(LoginRequiredMixin, UpdateView):
    model = Film
    form_class = FilmDevelopForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class FilmArchive(LoginRequiredMixin, UpdateView):
    model = Film
    form_class = FilmArchiveForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Film, owner=self.request.user, id_owner=self.kwargs['id_owner'])


@method_decorator(cache_control(private=True), name='dispatch')
class TeleconverterList(LoginRequiredMixin, PagedFilteredTableView):
    model = Teleconverter
    table_class = TeleconverterTable
    filterset_class = TeleconverterFilter
    formhelper_class = TeleconverterFormHelper


@method_decorator(cache_control(private=True), name='dispatch')
class TeleconverterDetail(LoginRequiredMixin, generic.DetailView):
    model = Teleconverter

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Teleconverter, owner=self.request.user, id_owner=self.kwargs['id_owner'])


class TeleconverterCreate(LoginRequiredMixin, CreateView):
    model = Teleconverter
    form_class = TeleconverterForm
    template_name = 'create.html'


class TeleconverterUpdate(LoginRequiredMixin, UpdateView):
    model = Teleconverter
    form_class = TeleconverterForm
    template_name = 'update.html'

    # Restrict to objects we own
    def get_object(self):
        return get_object_or_404(Teleconverter, owner=self.request.user, id_owner=self.kwargs['id_owner'])

class TeleconverterModelList(PagedFilteredTableView):
    model = TeleconverterModel
    table_class = TeleconverterModelTable
    filterset_class = TeleconverterModelFilter
    formhelper_class = TeleconverterModelFormHelper


class TeleconverterModelDetail(generic.DetailView):
    model = TeleconverterModel


class TeleconverterModelCreate(LoginRequiredMixin, CreateView):
    model = TeleconverterModel
    form_class = TeleconverterModelForm
    template_name = 'create.html'


class TeleconverterModelUpdate(LoginRequiredMixin, UpdateView):
    model = TeleconverterModel
    form_class = TeleconverterModelForm
    template_name = 'update.html'


class TonerList(PagedFilteredTableView):
    model = Toner
    table_class = TonerTable
    filterset_class = TonerFilter
    formhelper_class = TonerFormHelper


class TonerDetail(generic.DetailView):
    model = Toner

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        # Find similar objects of the same type
        similarobjects = self.get_object().tags.similar_objects()
        items = []
        for index, item in zip(range(10), similarobjects): # pylint: disable=unused-variable
            if type(item) == type(self.get_object()):  # pylint: disable=unidiomatic-typecheck
                detailitem = get_object_or_404(type(item), pk=item.pk)
                items.append(detailitem)
        context['related'] = items

        if self.get_object().history.all():
            context['history'] = self.get_object().history.all()

        return context


class TonerCreate(LoginRequiredMixin, CreateView):
    model = Toner
    form_class = TonerForm
    template_name = 'create.html'


class TonerUpdate(LoginRequiredMixin, UpdateView):
    model = Toner
    form_class = TonerForm
    template_name = 'update.html'


class StatsView(TemplateView):
    template_name = "stats.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        stats = [
            {
                'image': "svg/camera.svg",
                'url': reverse('schema:cameramodel-list'),
                'item': "camera models in CameraHub",
                'value': CameraModel.objects.count,
            },
            {
                'image': "svg/lens.svg",
                'url': reverse('schema:lensmodel-list'),
                'item': "lens models in CameraHub",
                'value': LensModel.objects.count,
            },
            {
                'image': "svg/film.svg",
                'url': reverse('schema:filmstock-list'),
                'item': "film stocks in CameraHub",
                'value': FilmStock.objects.count
            },
            {
                'image': "svg/manufacturer.svg",
                'url': reverse('schema:manufacturer-list'),
                'item': "manufacturers in CameraHub",
                'value': Manufacturer.objects.count
            },
            {
                'image': "svg/person.svg",
                'item': "users on CameraHub",
                'value': get_user_model().objects.count
            },
        ]

        oldestcamera = CameraModel.objects.filter(
            introduced__isnull=False).order_by('introduced').first()
        if oldestcamera:
            stats.append(
                {
                    'image': "svg/vintagecamera.svg",
                    'url': oldestcamera.get_absolute_url,
                    'item': "oldest camera on CameraHub",
                    'subheading': oldestcamera.introduced,
                    'value': oldestcamera,
                }
            )

        heaviestcamera = CameraModel.objects.filter(
            weight__isnull=False).order_by('weight').last()
        if heaviestcamera:
            stats.append(
                {
                    'image': "svg/bigcamera.svg",
                    'url': heaviestcamera.get_absolute_url,
                    'item': "heaviest camera on CameraHub",
                    'subheading': str(heaviestcamera.weight) + 'g',
                    'value': heaviestcamera
                }
            )

        longestlens = LensModel.objects.filter(
            max_focal_length__isnull=False).order_by('max_focal_length').last()
        if longestlens:
            stats.append(
                {
                    'image': "svg/lens.svg",
                    'url': longestlens.get_absolute_url,
                    'item': "longest lens on CameraHub",
                    'subheading': str(longestlens.max_focal_length) + 'mm',
                    'value': longestlens
                }
            )

        fastestlens = LensModel.objects.filter(
            max_aperture__isnull=False).order_by('max_aperture').first()
        if fastestlens:
            stats.append(
                {
                    'image': "svg/teleconverter.svg",
                    'url': fastestlens.get_absolute_url,
                    'item': "fastest lens on CameraHub",
                    'subheading': 'f/' + str(fastestlens.max_aperture),
                    'value': fastestlens
                }
            )

        stats.append(
            {
                'image': "svg/weight.svg",
                'url': reverse('schema:cameramodel-list'),
                'item': "total weight of all cameras in CameraHub",
                'value': str(round((CameraModel.objects.all().aggregate(totalweight=Sum('weight'))['totalweight'] or 0.00)/1000)) + 'kg',
            }
        )

        stats.append(
            {
                'image': "svg/ruler.svg",
                'url': reverse('schema:lensmodel-list'),
                'item': "total length of all lenses in CameraHub, laid end to end",
                'value': str(round((LensModel.objects.all().aggregate(totallength=Sum('length'))['totallength'] or 0.00)/1000)) + 'm',
            }
        )

        stats.append(
            {
                'image': "svg/camera.svg",
                'url': reverse('schema:camera-list'),
                'item': "cameras in user collections on CameraHub",
                'value': Camera.objects.count,
            }
        )

        stats.append(
            {
                'image': "svg/film.svg",
                'url': reverse('schema:camera-list'),
                'item': "total length of exposed film in CameraHub",
                'value': str(round((Negative.objects.all().aggregate(totallength=Sum('film__camera__cameramodel__negative_size__width'))['totallength'] or 0.00)/1000 )) + 'm',
            }
        )

        context['stats'] = stats
        context['title'] = "Public stats"
        return context


@method_decorator(cache_control(private=True), name='dispatch')
class MyStatsView(LoginRequiredMixin, TemplateView):
    template_name = "stats.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        stats = [
            {
                'image': "svg/camera.svg",
                'url': reverse('schema:camera-list'),
                'item': "cameras in your collection right now",
                'value': Camera.objects.filter(owner=self.request.user, own=True).count,
            },
            {
                'image': "svg/camera.svg",
                'url': reverse('schema:camera-list'),
                'item': "cameras ever in your collection",
                'value': Camera.objects.filter(owner=self.request.user).count,
            },
            {
                'image': "svg/lens.svg",
                'url': reverse('schema:lens-list'),
                'item': "lenses in your collection right now",
                'value': Lens.objects.filter(owner=self.request.user, own=True).count,
            },
            {
                'image': "svg/lens.svg",
                'url': reverse('schema:lens-list'),
                'item': "lenses ever in your collection",
                'value': Lens.objects.filter(owner=self.request.user).count,
            },
            {
                'image': "svg/film.svg",
                'url': reverse('schema:film-list'),
                'item': "films in your collection",
                'value': Film.objects.filter(owner=self.request.user).count,
            },
            {
                'image': "svg/negative.svg",
                'url': reverse('schema:negative-list'),
                'item': "negatives in your collection",
                'value': Negative.objects.filter(owner=self.request.user).count,
            },
            {
                'image': "svg/print.svg",
                'url': reverse('schema:print-list'),
                'item': "prints in your collection",
                'value': Print.objects.filter(owner=self.request.user).count,
            },
            {
                'image': "svg/film.svg",
                'url': reverse('schema:camera-list'),
                'item': "total length of exposed film in your collection",
                'value': str(round((Negative.objects.filter(owner=self.request.user).aggregate(totallength=Sum('film__camera__cameramodel__negative_size__width'))['totallength'] or 0.00)/1000 )) + 'm',
            },
            {
                'image': "svg/percent.svg",
                'url': reverse('schema:camera-list'),
                'item': "percentage of camera models you've owned",
                'value': str(round(100*(int(Camera.objects.filter(owner=self.request.user).values('cameramodel').distinct().count())/int(CameraModel.objects.count())))) + '%',
            },
            {
                'image': "svg/percent.svg",
                'url': reverse('schema:camera-list'),
                'item': "percentage of lens models you've owned",
                'value': str(round(100*(int(Lens.objects.filter(owner=self.request.user).values('lensmodel').distinct().count())/int(LensModel.objects.count())))) + '%',
            },
            {
                'image': "svg/ownership.svg",
                'url': reverse('schema:camera-list'),
                'item': "net spent on cameras",
                'value': str(round(((Camera.objects.filter(owner=self.request.user).aggregate(diff=(Sum('cost') - Sum('lost_price')))['diff'] or 0)), 2)),
            },
            {
                'image': "svg/ownership.svg",
                'url': reverse('schema:lens-list'),
                'item': "net spent on lenses",
                'value': str(round(((Lens.objects.filter(owner=self.request.user).aggregate(diff=(Sum('cost') - Sum('lost_price')))['diff'] or 0)), 2)),
            }
        ]

        context['stats'] = stats
        context['title'] = "Private stats"
        return context


class SearchView(SearchMixin, generic.ListView):

    """View that performs a search and returns the search results."""

    template_name = "search.html"


class TagList(ListView):
    model = Tag
    template_name = 'tag-list.html'


class TagDetail(generic.DetailView):
    model = Tag
    template_name = 'tag-detail.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        taggeditems = self.get_object().taggit_taggeditem_items.all()

        items = []
        for item in taggeditems:
            model = apps.get_model('schema', item.content_type.model)
            detailitem = get_object_or_404(model, pk=item.object_id)
            items.append(detailitem)

        context['taggeditems'] = items
        return context


class TagAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        # Don't forget to filter out results depending on the visitor !
        if not self.request.user.is_authenticated:
            return Tag.objects.none()

        qs = Tag.objects.all().order_by('name')

        if self.q:
            qs = qs.filter(name__istartswith=self.q)

        return qs



# Generated by Django 2.2.4 on 2019-10-31 11:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0013_delete_filteradapter'),
    ]

    operations = [
        migrations.RenameField(
            model_name='repair',
            old_name='description',
            new_name='detail',
        ),
    ]



# Generated by Django 2.2.14 on 2020-08-27 20:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0078_auto_20200825_2104'),
    ]

    operations = [
        migrations.AddField(
            model_name='cameramodel',
            name='fastest_shutter_speed',
            field=models.ForeignKey(blank=True, help_text='Fastest shutter speed available on this camera', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='fastest_shutter_speed', to='schema.ShutterSpeed'),
        ),
        migrations.AddField(
            model_name='cameramodel',
            name='slowest_shutter_speed',
            field=models.ForeignKey(blank=True, help_text='Slowest shutter speed available on this camera', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='slowest_shutter_speed', to='schema.ShutterSpeed'),
        ),
        migrations.AddField(
            model_name='historicalcameramodel',
            name='fastest_shutter_speed',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='Fastest shutter speed available on this camera', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='schema.ShutterSpeed'),
        ),
        migrations.AddField(
            model_name='historicalcameramodel',
            name='slowest_shutter_speed',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='Slowest shutter speed available on this camera', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='schema.ShutterSpeed'),
        ),
    ]



import sys
from django.forms import ModelForm
from django.db.models import Q
from django_currentuser.middleware import get_current_user
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Fieldset, Submit, Div, Hidden, HTML
from crispy_forms.bootstrap import FormActions, AppendedText, InlineCheckboxes, PrependedText, TabHolder, Tab
from dal import autocomplete
from bootstrap_datepicker_plus import DatePickerInput, DateTimePickerInput, YearPickerInput, MonthPickerInput, TimePickerInput

from schema.models import Accessory, Archive, Battery, BulkFilm, Camera, CameraModel, Developer, Enlarger, EnlargerModel, FilmStock, Filter
from schema.models import Flash, FlashModel, Format, Lens, LensModel, Manufacturer
from schema.models import Mount, MountAdapter, NegativeSize, Order, PaperStock, Person, Print
from schema.models import Process, Scan, Negative, Film, Teleconverter, TeleconverterModel, Toner

FormActionButtons = Layout(
    FormActions(
        Submit('save', 'Save')
    )
)


class AccessoryForm(ModelForm):
    class Meta:
        model = Accessory
        fields = ['manufacturer', 'model', 'type', 'acquired', 'cost', 'lost',
                  'lost_price', 'camera_model_compatibility', 'lens_model_compatibility']
        widgets = {
            'acquired': DatePickerInput(format='%Y-%m-%d'),
            'lost': DatePickerInput(format='%Y-%m-%d'),
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('manufacturer')
            fields.remove('camera_model_compatibility')
            fields.remove('lens_model_compatibility')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'manufacturer',
                     'model',
                     'type',
                     ),
            Fieldset('Compatibility',
                     'camera_model_compatibility',
                     'lens_model_compatibility',
                     ),
            Fieldset('Ownership',
                     'acquired',
                     'cost',
                     'lost',
                     'lost_price',
                     ),
            FormActionButtons
        )


class ArchiveForm(ModelForm):
    class Meta:
        model = Archive
        fields = ['name', 'type', 'max_width',
                  'max_height', 'location', 'storage', 'sealed']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Div(
                'name',
                'type',
                AppendedText('max_width', '"'),
                AppendedText('max_height', '"'),
                'location',
                'storage',
                'sealed',
            ),
            FormActionButtons
        )


class BatteryForm(ModelForm):
    class Meta:
        model = Battery
        fields = ['name', 'voltage', 'chemistry', 'compatible_with']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'name',
                     AppendedText('voltage', 'V'),
                     'chemistry',
                     ),
            Fieldset('Compatibility',
                     'compatible_with',
                     ),
            FormActionButtons
        )


class BulkFilmForm(ModelForm):
    class Meta:
        model = BulkFilm
        fields = ['format', 'filmstock', 'length', 'finished', 'purchase_date',
                  'cost', 'source', 'batch', 'expiry']
        widgets = {
            'purchase_date': DatePickerInput(format='%Y-%m-%d'),
            'expiry': MonthPickerInput(format='%Y-%m-01'),
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('filmstock')
            fields.remove('format')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'format',
                     'filmstock',
                     AppendedText('length', 'm'),
                     'finished',
                     'batch',
                     'expiry',
                     ),
            Fieldset('Ownership',
                     'purchase_date',
                     'cost',
                     'source',
                     ),
            FormActionButtons
        )


class CameraForm(ModelForm):
    class Meta:
        model = Camera
        fields = ['cameramodel', 'acquired', 'cost', 'source', 'serial', 'datecode',
                  'manufactured', 'own', 'notes', 'lost', 'lost_price', 'condition', 'condition_notes']
        widgets = {
            'acquired': DatePickerInput(format='%Y-%m-%d'),
            'lost': DatePickerInput(format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'cameramodel',
                     'serial',
                     'datecode',
                     'manufactured',
                     'notes',
                     'condition',
                     'condition_notes',
                     ),
            Fieldset('Ownership',
                     'own',
                     'acquired',
                     'cost',
                     'source',
                     'lost',
                     'lost_price',
                     ),
            FormActionButtons
        )


class CameraSellForm(ModelForm):
    class Meta:
        model = Camera
        fields = ['lost', 'lost_price', 'own']
        widgets = {
            'lost': DatePickerInput(format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Div(
                Hidden('own', False),
                'lost',
                'lost_price',
            ),
            FormActionButtons
        )


class CameraModelForm(autocomplete.FutureModelForm):
    class Meta:
        model = CameraModel
        fields = '__all__'
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete'),
            'introduced': YearPickerInput(format='%Y'),
            'discontinued': YearPickerInput(format='%Y'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)

        self.helper.layout = Layout(
            Fieldset('Summary',
                     'manufacturer',
                     'model',
                     'other_names',
                     'disambiguation',
                     'introduced',
                     'discontinued',
                     'format',
                     'negative_size',
                     ),
            Div(
                TabHolder(
                    Tab('Interchangeable lens',
                        Fieldset('Lens mount',
                                 'mount',
                                 ),
                        ),
                    Tab('Fixed lens',
                        Fieldset('Lens',
                                 'lens_manufacturer',
                                 'lens_model_name',
                                 ),
                        Fieldset('Optics',
                                 'zoom',
                                 AppendedText(
                                     'min_focal_length', 'mm'),
                                 AppendedText('max_focal_length', 'mm'),
                                 PrependedText('max_aperture', 'f/'),
                                 PrependedText('min_aperture', 'f/'),
                                 AppendedText('closest_focus', 'm'),
                                 'elements',
                                 'groups',
                                 AppendedText(
                                     'nominal_min_angle_diag', '&deg;'),
                                 AppendedText(
                                     'nominal_max_angle_diag', '&deg;'),
                                 'aperture_blades',
                                 'coating',
                                 AppendedText('magnification', '&times;'),
                                 ),
                        Fieldset('Physical',
                                 AppendedText('filter_thread', 'mm'),
                                 'hood',
                                 ),
                        ),
                ),
                css_class="border",
            ),
            Fieldset('Physical',
                     'body_type',
                     AppendedText('weight', 'g'),
                     ),
            Fieldset('Focus',
                     'focus_type',
                     'af_points',
                     AppendedText('viewfinder_coverage', '%'),
                     ),
            Fieldset('Metering',
                     'metering',
                     'metering_type',
                     'min_iso',
                     'max_iso',
                     'dx_code',
                     'meter_min_ev',
                     'meter_max_ev',
                     InlineCheckboxes('metering_modes'),
                     InlineCheckboxes('exposure_programs'),
                     ),
            Fieldset('Shutter',
                     'shutter_type',
                     'shutter_model',
                     'fastest_shutter_speed',
                     'slowest_shutter_speed',
                     'bulb',
                     'time',
                     ),
            Fieldset('Film transport',
                     'internal_power_drive',
                     AppendedText('continuous_fps', 'fps'),
                     'external_power_drive',
                     ),
            Fieldset('Power',
                     'battery_qty',
                     'battery_type',
                     ),
            Fieldset('Flash',
                     'int_flash',
                     AppendedText('int_flash_gn', 'm'),
                     'ext_flash',
                     'pc_sync',
                     'shoe',
                     'x_sync',
                     ),
            Fieldset('Features',
                     'dof_preview',
                     'mirror_lockup',
                     'tripod',
                     'self_timer',
                     'date_imprint',
                     'cable_release',
                     'interchangeable_backs',
                     'interchangeable_finders',
                     'strap_lugs',
                     'multiple_exposures',
                     ),
            Fieldset('Misc',
                     'notes',
                     'tags',
                     'link',
                     'image',
                     'image_attribution',
                     'image_attribution_link',
                     ),
            FormActionButtons
        )


class DeveloperForm(ModelForm):
    class Meta:
        model = Developer
        fields = ['manufacturer', 'name', 'for_paper',
                  'for_film', 'chemistry', 'tags']
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete')
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('manufacturer')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'manufacturer',
                     'name',
                     'for_paper',
                     'for_film',
                     'chemistry',
                     ),
            Fieldset('Meta',
                     'tags',
                     ),
            FormActionButtons
        )


class EnlargerModelForm(ModelForm):
    class Meta:
        model = EnlargerModel
        fields = ['manufacturer', 'model', 'disambiguation', 'negative_size',
                  'type', 'light_source', 'introduced', 'discontinued', 'tags', 'image', 'image_attribution', 'image_attribution_link']
        widgets = {
            'introduced': YearPickerInput(format='%Y'),
            'discontinued': YearPickerInput(format='%Y'),
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete')
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('manufacturer')
            fields.remove('negative_size')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'manufacturer',
                     'model',
                     'disambiguation',
                     'negative_size',
                     'type',
                     'light_source',
                     'introduced',
                     'discontinued',
                     ),
            Fieldset('Meta',
                     'tags',
                     'image',
                     'image_attribution',
                     'image_attribution_link',
                     ),
            FormActionButtons
        )


class EnlargerForm(ModelForm):
    class Meta:
        model = Enlarger
        fields = ['enlargermodel', 'serial', 'own',
                  'acquired', 'cost', 'lost', 'lost_price']
        widgets = {
            'acquired': DatePickerInput(format='%Y-%m-%d'),
            'lost': DatePickerInput(format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'enlargermodel',
                     'serial',
                     ),
            Fieldset('Ownership',
                     'own',
                     'acquired',
                     'cost',
                     'lost',
                     'lost_price',
                     ),
            FormActionButtons
        )


class FilmStockForm(ModelForm):
    class Meta:
        model = FilmStock
        fields = ['name', 'manufacturer', 'iso',
                  'colour', 'panchromatic', 'process', 'tags']
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete')
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('manufacturer')
            fields.remove('process')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'name',
                     'manufacturer',
                     'iso',
                     ),
            Fieldset('Features',
                     'colour',
                     'panchromatic',
                     'process',
                     ),
            Fieldset('Meta',
                     'tags',
                     ),
            FormActionButtons
        )


class FilterForm(ModelForm):
    class Meta:
        model = Filter
        fields = ['type', 'shortname', 'attenuation']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            'type',
            'shortname',
            'attenuation',
            FormActionButtons
        )


class FlashModelForm(ModelForm):
    class Meta:
        model = FlashModel
        fields = ['manufacturer', 'model', 'disambiguation', 'guide_number', 'gn_info', 'battery_powered', 'pc_sync', 'hot_shoe', 'light_stand', 'battery_type',
                  'battery_qty', 'manual_control', 'swivel_head', 'tilt_head', 'zoom', 'ttl', 'trigger_voltage', 'tags', 'image', 'image_attribution', 'image_attribution_link']
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete')
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('manufacturer')
            fields.remove('battery_type')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'manufacturer',
                     'model',
                     'disambiguation',
                     AppendedText('guide_number', 'm'),
                     'gn_info',
                     AppendedText('trigger_voltage', 'V'),
                     ),
            Fieldset('Features',
                     'pc_sync',
                     'hot_shoe',
                     'light_stand',
                     'manual_control',
                     'swivel_head',
                     'tilt_head',
                     'zoom',
                     'ttl',
                     ),
            Fieldset('Power',
                     'battery_powered',
                     'battery_type',
                     'battery_qty',
                     ),
            Fieldset('Meta',
                     'tags',
                     'image',
                     'image_attribution',
                     'image_attribution_link',
                     ),
            FormActionButtons
        )


class FlashForm(ModelForm):
    class Meta:
        model = Flash
        fields = ['flashmodel', 'serial', 'own',
                  'acquired', 'cost', 'lost', 'lost_price']
        widgets = {
            'acquired': DatePickerInput(format='%Y-%m-%d'),
            'lost': DatePickerInput(format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'flashmodel',
                     'serial',
                     ),
            Fieldset('Ownership',
                     'own',
                     'acquired',
                     'cost',
                     'lost',
                     'lost_price',
                     ),
            FormActionButtons
        )


class FormatForm(ModelForm):
    class Meta:
        model = Format
        fields = ['format', 'negative_size']
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('negative_size')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            'format',
            'negative_size',
            FormActionButtons
        )


class LensForm(ModelForm):
    class Meta:
        model = Lens
        fields = ['lensmodel', 'serial', 'date_code', 'manufactured', 'acquired', 'cost',
                  'notes', 'own', 'lost', 'lost_price', 'source', 'condition', 'condition_notes']
        widgets = {
            'acquired': DatePickerInput(format='%Y-%m-%d'),
            'lost': DatePickerInput(format='%Y-%m-%d'),
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('lensmodel')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'lensmodel',
                     'serial',
                     'date_code',
                     'manufactured',
                     'notes',
                     'condition',
                     'condition_notes',
                     ),
            Fieldset('Ownership',
                     'own',
                     'acquired',
                     'cost',
                     'source',
                     'lost',
                     'lost_price',
                     ),
            FormActionButtons
        )


class LensSellForm(ModelForm):
    class Meta:
        model = Lens
        fields = ['own', 'lost', 'lost_price']
        widgets = {
            'lost': DatePickerInput(format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Div(
                Hidden('own', False),
                'lost',
                'lost_price',
            ),
            FormActionButtons
        )


class LensModelForm(ModelForm):
    class Meta:
        model = LensModel
        fields = ['manufacturer', 'model', 'disambiguation', 'mount', 'introduced', 'discontinued', 'zoom', 'min_focal_length', 'max_focal_length', 'max_aperture', 'min_aperture', 'closest_focus', 'elements', 'groups', 'nominal_min_angle_diag', 'nominal_max_angle_diag', 'lens_type', 'image_circle', 'aperture_blades',
                  'coating', 'autofocus', 'perspective_control', 'magnification', 'negative_size', 'weight', 'length', 'diameter', 'filter_thread', 'hood', 'shutter_model', 'notes', 'tags', 'link', 'image', 'image_attribution', 'image_attribution_link', 'diagram', 'diagram_attribution', 'diagram_attribution_link']
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete'),
            'introduced': YearPickerInput(format='%Y'),
            'discontinued': YearPickerInput(format='%Y'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'manufacturer',
                     'model',
                     'disambiguation',
                     'mount',
                     'introduced',
                     'discontinued',
                     ),
            Fieldset('Optics',
                     'zoom',
                     AppendedText('min_focal_length', 'mm'),
                     AppendedText('max_focal_length', 'mm'),
                     PrependedText('max_aperture', 'f/'),
                     PrependedText('min_aperture', 'f/'),
                     AppendedText('closest_focus', 'm'),
                     'elements',
                     'groups',
                     AppendedText('nominal_min_angle_diag', '&deg;'),
                     AppendedText('nominal_max_angle_diag', '&deg;'),
                     'lens_type',
                     AppendedText('magnification', '&times;'),
                     'negative_size',
                     'aperture_blades',
                     AppendedText('image_circle', 'mm'),
                     'coating',
                     'diagram',
                     'diagram_attribution',
                     'diagram_attribution_link',
                     ),
            Fieldset('Features',
                     'autofocus',
                     'perspective_control',
                     'hood',
                     'shutter_model',
                     ),
            Fieldset('Physical',
                     AppendedText('weight', 'g'),
                     AppendedText('length', 'mm'),
                     AppendedText('diameter', 'mm'),
                     AppendedText('filter_thread', 'mm'),
                     ),
            Fieldset('Misc',
                     'notes',
                     'link',
                     'image',
                     'image_attribution',
                     'image_attribution_link',
                     ),
            Fieldset('Meta',
                     'tags',
                     ),
            FormActionButtons
        )


class ManufacturerForm(ModelForm):
    class Meta:
        model = Manufacturer
        fields = ['name', 'city', 'country', 'link',
                  'founded', 'dissolved', 'tags']
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete'),
            'founded': YearPickerInput(format='%Y'),
            'dissolved': YearPickerInput(format='%Y'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'name',
                     'city',
                     'country',
                     'link',
                     'founded',
                     'dissolved',
                     ),
            Fieldset('Meta',
                     'tags',
                     ),
            FormActionButtons
        )


class MountForm(ModelForm):
    class Meta:
        model = Mount
        fields = ['mount', 'shutter_in_lens', 'type',
                  'purpose', 'notes', 'manufacturer', 'tags']
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete')
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('manufacturer')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'mount',
                     'shutter_in_lens',
                     'type',
                     'purpose',
                     'notes',
                     'manufacturer',
                     ),
            Fieldset('Meta',
                     'tags',
                     ),
            FormActionButtons
        )


class MountAdapterForm(ModelForm):
    class Meta:
        model = MountAdapter
        fields = ['camera_mount', 'lens_mount',
                  'has_optics', 'infinity_focus', 'notes']
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('camera_mount')
            fields.remove('lens_mount')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            'camera_mount',
            'lens_mount',
            'has_optics',
            'infinity_focus',
            'notes',
            FormActionButtons
        )


class NegativeSizeForm(ModelForm):
    class Meta:
        model = NegativeSize
        fields = [
            'name',
            'width',
            'height',
        ]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            'name',
            AppendedText('width', 'mm'),
            AppendedText('height', 'mm'),
            FormActionButtons
        )


class OrderForm(ModelForm):
    class Meta:
        model = Order
        fields = ['negative', 'width', 'height',
                  'added', 'printed', 'print', 'recipient']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['negative'].queryset = Negative.objects.filter(
            owner=get_current_user())
        self.fields['print'].queryset = Print.objects.filter(
            owner=get_current_user())
        self.fields['recipient'].queryset = Person.objects.filter(
            owner=get_current_user())

        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            'negative',
            AppendedText('width', '"'),
            AppendedText('height', '"'),
            'added',
            'printed',
            'print',
            'recipient',
            FormActionButtons
        )


class PaperStockForm(ModelForm):
    class Meta:
        model = PaperStock
        fields = ['name', 'manufacturer',
                  'resin_coated', 'colour', 'finish', 'tags']
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete')
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('manufacturer')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'name',
                     'manufacturer',
                     'resin_coated',
                     'colour',
                     'finish',
                     ),
            Fieldset('Meta',
                     'tags',
                     ),
            FormActionButtons
        )


class PersonForm(ModelForm):
    class Meta:
        model = Person
        fields = ['name', 'type']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            'name',
            'type',
            FormActionButtons
        )


class PrintForm(ModelForm):
    class Meta:
        model = Print
        fields = ['negative', 'date', 'paper_stock', 'height', 'width', 'aperture', 'exposure_time', 'filtration_grade', 'development_time',
                  'toner', 'own', 'location', 'sold_price', 'enlarger', 'lens', 'developer', 'fine', 'notes', 'archive', 'printer']
        widgets = {
            'date': DatePickerInput(format='%Y-%m-%d'),
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('paper_stock')
            fields.remove('developer')
            fields.remove('toner')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['negative'].queryset = Negative.objects.filter(
            owner=get_current_user())
        self.fields['enlarger'].queryset = Enlarger.objects.filter(
            owner=get_current_user())
        self.fields['lens'].queryset = Lens.objects.filter(
            owner=get_current_user())
        self.fields['archive'].queryset = Archive.objects.filter(
            owner=get_current_user(), type='Print', sealed=False)
        self.fields['printer'].queryset = Person.objects.filter(
            owner=get_current_user())
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'negative',
                     'date',
                     'paper_stock',
                     AppendedText('width', '"'),
                     AppendedText('height', '"'),
                     'fine',
                     ),
            Fieldset('Exposure',
                     'enlarger',
                     'lens',
                     PrependedText('aperture', 'f/'),
                     'exposure_time',
                     'filtration_grade',
                     ),
            Fieldset('Development',
                     'developer',
                     'development_time',
                     'toner',
                     'notes',
                     'printer',
                     ),
            Fieldset('Ownership',
                     'own',
                     'location',
                     'sold_price',
                     'archive',
                     ),
            FormActionButtons
        )


class PrintArchiveForm(ModelForm):
    class Meta:
        model = Print
        fields = ['archive']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['archive'].queryset = Archive.objects.filter(type='Print', owner=get_current_user(), sealed=False)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Archive this print',
                     'archive',
                     ),
            Hidden('status', 'Archived'),
            FormActionButtons
        )

class ProcessForm(ModelForm):
    class Meta:
        model = Process
        fields = ['name', 'colour', 'positive']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            'name',
            'colour',
            'positive',
            FormActionButtons
        )


class ScanForm(ModelForm):
    class Meta:
        model = Scan
        fields = ['negative', 'print', 'filename', 'date']
        widgets = {
            'date': DatePickerInput(format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['negative'].queryset = Negative.objects.filter(
            owner=get_current_user())
        self.fields['print'].queryset = Print.objects.filter(
            owner=get_current_user())
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            'negative',
            'print',
            'filename',
            'date',
            FormActionButtons
        )


class NegativeForm(ModelForm):
    class Meta:
        model = Negative
        fields = ['film', 'frame', 'caption', 'date', 'lens', 'shutter_speed', 'aperture', 'filter', 'teleconverter', 'notes',
                  'mount_adapter', 'focal_length', 'location', 'flash', 'metering_mode', 'exposure_program', 'photographer', 'copy_of']
        widgets = {
            'date': DateTimePickerInput(format='%Y-%m-%d %H:%M', options={"sideBySide": True}),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['film'].queryset = Film.objects.filter(
            owner=get_current_user()).exclude(status='Available')
        self.fields['lens'].queryset = Lens.objects.filter(
            owner=get_current_user())
        self.fields['mount_adapter'].queryset = MountAdapter.objects.filter(
            owner=get_current_user())
        self.fields['photographer'].queryset = Person.objects.filter(
            owner=get_current_user())
        self.fields['copy_of'].queryset = Negative.objects.filter(
            owner=get_current_user())
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'film',
                     'frame',
                     'caption',
                     'date',
                     ),
            Fieldset('Exposure',
                     'lens',
                     AppendedText('focal_length', 'mm'),
                     'shutter_speed',
                     PrependedText('aperture', 'f/'),
                     'filter',
                     'teleconverter',
                     'mount_adapter',
                     'flash',
                     'metering_mode',
                     'exposure_program',
                     ),
            Fieldset('Misc',
                     'photographer',
                     'copy_of',
                     'notes',
                     'location',
                     ),
            FormActionButtons
        )


class FilmForm(ModelForm):
    class Meta:
        model = Film
        fields = ['filmstock', 'exposed_at', 'format', 'status', 'date_loaded', 'date_processed', 'camera', 'title', 'frames', 'developer', 'developer_previous_uses', 'development_time',
                  'development_temperature', 'development_compensation', 'development_notes', 'bulk_film', 'bulk_film_loaded', 'film_batch', 'expiry_date', 'purchase_date', 'price', 'processed_by', 'archive']
        widgets = {
            'date_loaded': DatePickerInput(format='%Y-%m-%d'),
            'date_processed': DatePickerInput(format='%Y-%m-%d'),
            'purchase_date': DatePickerInput(format='%Y-%m-%d'),
            'bulk_film_loaded': DatePickerInput(format='%Y-%m-%d'),
            'expiry_date': MonthPickerInput(format='%Y-%m-01'),
            'dev_time': TimePickerInput(format='%H:%M:%S',
                                        options={
                                            "useCurrent": False,
                                            "showTodayButton": False,
                                        }),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['camera'].queryset = Camera.objects.filter(
            owner=get_current_user())
        self.fields['bulk_film'].queryset = BulkFilm.objects.filter(
            owner=get_current_user())
        self.fields['processed_by'].queryset = Person.objects.filter(
            owner=get_current_user())
        self.fields['archive'].queryset = Archive.objects.filter(
            owner=get_current_user())
        self.fields['bulk_film'].queryset = BulkFilm.objects.filter(
            owner=get_current_user(), finished=False)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'title',
                     'status',
                     ),
            Fieldset('Filmstock',
                     'filmstock',
                     'format',
                     'bulk_film',
                     'bulk_film_loaded',
                     'film_batch',
                     'expiry_date'
                     ),
            Fieldset('Exposure',
                     'camera',
                     'date_loaded',
                     'frames',
                     'exposed_at',
                     ),
            Fieldset('Development',
                     'developer',
                     'date_processed',
                     'developer_previous_uses',
                     'development_time',
                     AppendedText('development_temperature', '&deg;C'),
                     PrependedText('development_compensation', 'N'),
                     'development_notes',
                     'processed_by',
                     ),
            Fieldset('Archive',
                     'archive',
                     ),
            Fieldset('Ownership',
                     'purchase_date',
                     'price',
                     ),
            FormActionButtons
        )


class FilmAddForm(ModelForm):
    class Meta:
        model = Film
        fields = ['filmstock', 'format', 'frames', 'film_batch', 'expiry_date',
                  'purchase_date', 'price', 'bulk_film', 'bulk_film_loaded', 'status']
        widgets = {
            'purchase_date': DatePickerInput(format='%Y-%m-%d'),
            'expiry_date': MonthPickerInput(format='%Y-%m-01'),
            'bulk_film_loaded': DatePickerInput(format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['bulk_film'].queryset = BulkFilm.objects.filter(
            owner=get_current_user(), finished=False)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Add a new film to your collection',
                     'filmstock',
                     'format',
                     'frames',
                     Div(
                         TabHolder(
                             Tab('Single film',
                                 HTML(
                                     "<p>Choose Single Film for regular roll or sheet films</p>"),
                                 'film_batch',
                                 'expiry_date',
                                 'purchase_date',
                                 'price',
                                 ),
                             Tab('Bulk film',
                                 HTML(
                                     "<p>Choose Bulk Film for film that has been cut from a bulk roll</p>"),
                                 'bulk_film',
                                 'bulk_film_loaded',
                                 ),
                         ),
                         css_class="border",
                     ),
                     ),
            Hidden('status', 'Available'),
            FormActionButtons
        )


class FilmLoadForm(ModelForm):
    class Meta:
        model = Film
        fields = ['camera', 'title', 'exposed_at',
                  'date_loaded', 'frames', 'status']
        widgets = {
            'date_loaded': DatePickerInput(format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['camera'].queryset = Camera.objects.filter(
            owner=get_current_user(), cameramodel__format=self.instance.format)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Load this film into a camera',
                     'camera',
                     'title',
                     'exposed_at',
                     'date_loaded',
                     'frames',
                     ),
            Hidden('status', 'Loaded'),
            FormActionButtons
        )


class FilmDevelopForm(ModelForm):
    class Meta:
        model = Film
        fields = ['date_processed', 'developer', 'developer_previous_uses', 'development_time',
                  'development_temperature', 'development_compensation', 'development_notes', 'processed_by', 'status']
        widgets = {
            'date_processed': DatePickerInput(format='%Y-%m-%d'),
            'dev_time': TimePickerInput(format='%H:%M:%S',
                                        options={
                                            "useCurrent": False,
                                            "showTodayButton": False,
                                        }),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['processed_by'].queryset = Person.objects.filter(
            owner=get_current_user())
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Develop this film',
                     'date_processed',
                     'developer',
                     'developer_previous_uses',
                     'development_time',
                     AppendedText('development_temperature', '&deg;C'),
                     PrependedText('development_compensation', 'N'),
                     'development_notes',
                     'processed_by',
                     ),
            Hidden('status', 'Developed'),
            FormActionButtons
        )


class FilmArchiveForm(ModelForm):
    class Meta:
        model = Film
        fields = ['archive', 'status']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['archive'].queryset = Archive.objects.filter(
            Q(type='Negative') | Q(type='Slide'), owner=get_current_user(), sealed=False)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Archive this film',
                     'archive',
                     ),
            Hidden('status', 'Archived'),
            FormActionButtons
        )


class TeleconverterForm(ModelForm):
    class Meta:
        model = Teleconverter
        fields = ['teleconvertermodel', 'serial', 'own',
                  'acquired', 'cost', 'lost', 'lost_price', ]
        widgets = {
            'acquired': DatePickerInput(format='%Y-%m-%d'),
            'lost': DatePickerInput(format='%Y-%m-%d'),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'teleconvertermodel',
                     'serial',
                     ),
            Fieldset('Ownership',
                     'own',
                     'acquired',
                     'cost',
                     'lost',
                     'lost_price',
                     ),
            FormActionButtons
        )


class TeleconverterModelForm(ModelForm):
    class Meta:
        model = TeleconverterModel
        fields = ['model', 'manufacturer', 'disambiguation', 'mount',
                  'factor', 'elements', 'groups', 'multicoated', 'tags', 'image', 'image_attribution', 'image_attribution_link']
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete')
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'manufacturer',
                     'model',
                     'disambiguation'
                     'mount',
                     ),
            Fieldset('Optics',
                     AppendedText('factor', '&times;'),
                     'elements',
                     'groups',
                     'multicoated',
                     ),
            Fieldset('Meta',
                     'tags',
                     'image',
                     'image_attribution',
                     'image_attribution_link',
                     ),
            FormActionButtons
        )


class TonerForm(ModelForm):
    class Meta:
        model = Toner
        fields = [
            'name',
            'manufacturer',
            'formulation',
            'stock_dilution',
            'tags',
        ]
        widgets = {
            'tags': autocomplete.TaggitSelect2('schema:tag-autocomplete')
        }
        if ('makemigrations' in sys.argv or 'migrate' in sys.argv or 'test' in sys.argv):
            fields.remove('manufacturer')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.layout = Layout(
            Fieldset('Summary',
                     'name',
                     'manufacturer',
                     'formulation',
                     'stock_dilution',
                     ),
            Fieldset('Meta',
                     'tags',
                     ),
            FormActionButtons
        )



from django.db import migrations

def zoom_ratio_lensmodel(apps, schema_editor):
    lensmodel = apps.get_model('schema', 'LensModel')
    for row in lensmodel.objects.filter(zoom=True, min_focal_length__isnull=False, max_focal_length__isnull=False):
        row.zoom_ratio = row.max_focal_length / row.min_focal_length
        row.save()

def zoom_ratio_cameramodel(apps, schema_editor):
    cameramodel = apps.get_model('schema', 'CameraModel')
    for row in cameramodel.objects.filter(zoom=True, min_focal_length__isnull=False, max_focal_length__isnull=False):
        row.zoom_ratio = row.max_focal_length / row.min_focal_length
        row.save()

class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0060_auto_20200527_1946'),
    ]

    operations = [
        migrations.RunPython(zoom_ratio_lensmodel),
        migrations.RunPython(zoom_ratio_cameramodel),
    ]



# Generated by Django 2.2.17 on 2020-12-11 20:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0113_auto_20201127_2102'),
    ]

    operations = [
        migrations.AddField(
            model_name='bulkfilm',
            name='finished',
            field=models.BooleanField(default=False, help_text='Whether this bulk film has been used up'),
        ),
        migrations.AddField(
            model_name='bulkfilm',
            name='length',
            field=models.PositiveIntegerField(blank=True, help_text='Length of bulk film roll', null=True),
        ),
    ]



# Generated by Django 2.2 on 2020-03-27 23:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0049_auto_20200327_2231'),
    ]

    operations = [
        migrations.RenameField(
            model_name='cameramodel',
            old_name='lens_model',
            new_name='lens_model_name',
        ),
        migrations.RemoveField(
            model_name='camera',
            name='lens',
        ),
        migrations.RemoveField(
            model_name='cameramodel',
            name='fixed_mount',
        ),
        migrations.RemoveField(
            model_name='cameramodel',
            name='lensmodel',
        ),
        migrations.RemoveField(
            model_name='lensmodel',
            name='fixed_mount',
        ),
    ]



# Generated by Django 2.2.4 on 2019-11-08 12:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0016_auto_20191107_2005'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cameramodel',
            name='metering_type',
            field=models.CharField(blank=True, choices=[('Cadmium sulphide CdS', 'Cadmium sulphide CdS'), ('Selenium', 'Selenium'), (
                'Silicon', 'Silicon')], help_text='Metering type used on this camera model', max_length=25, null=True),
        ),
        migrations.DeleteModel(
            name='MeteringType',
        ),
    ]



from __future__ import unicode_literals
from django.db import migrations, models


def addFlash(apps, schema_editor):
    flashes = apps.get_model('schema', 'flashmodel')
    for record in flashes.objects.all():
        record.save()


def addEnlarger(apps, schema_editor):
    enlargers = apps.get_model('schema', 'enlargermodel')
    for record in enlargers.objects.all():
        record.save()


def addTeleconverter(apps, schema_editor):
    teleconverters = apps.get_model('schema', 'teleconvertermodel')
    for record in teleconverters.objects.all():
        record.save()


class Migration(migrations.Migration):
    dependencies = [
        ('schema', '0111_auto_20201126_2251'),
    ]
    operations = [
        migrations.RunPython(
            addFlash, reverse_code=migrations.RunPython.noop
        ),
        migrations.RunPython(
            addEnlarger, reverse_code=migrations.RunPython.noop
        ),
        migrations.RunPython(
            addTeleconverter, reverse_code=migrations.RunPython.noop
        ),
    ]



# Generated by Django 2.2.4 on 2019-10-16 11:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('schema', '0011_auto_20191016_1035'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='projector',
            name='manufacturer',
        ),
        migrations.RemoveField(
            model_name='projector',
            name='mount',
        ),
        migrations.RemoveField(
            model_name='projector',
            name='negative_size',
        ),
        migrations.RemoveField(
            model_name='projector',
            name='owner',
        ),
        migrations.AlterField(
            model_name='accessory',
            name='type',
            field=models.CharField(choices=[('Battery grip', 'Battery grip'), ('Case', 'Case'), ('Film back', 'Film back'), ('Focusing screen', 'Focusing screen'), ('Lens hood', 'Lens hood'), ('Lens cap', 'Lens cap'), (
                'Power winder', 'Power winder'), ('Viewfinder', 'Viewfinder'), ('Rangefinder', 'Rangefinder'), ('Projector', 'Projector'), ('Light meter', 'Light meter')], help_text='Type of accessory', max_length=15),
        ),
        migrations.DeleteModel(
            name='LightMeter',
        ),
        migrations.DeleteModel(
            name='Projector',
        ),
    ]
